/*------------------------------------------------------*\

    Author:     Ide Nentjes
                ide.nentjes@consul.com
                http://www.consul.com

    Date:       April 17, 2001
    Revised:    April 17, 2001
    Name:       TreeView.h
    Version:    1.01

Program by the nefarious Ide Nentjes

This program may be freely copied and distributed without
restriction.  The one proviso is that full credit be given.

Instructions: 
 - include TreeView.h and TreeView.rul in your project.
 - initialize the bitmaps to be used with CtrlTreeIniBitmapFromFile.
 - Fill the Tree with either CtrlTreeFillFromList or CtrlTreeFillFromMedia.
 - IMPORTANT: Call DestroyWindow before EndDialog, or IS will cause a memory exception on runtime.

Notes:
 - See Sample.h and Sample.rul for examples on how to use the functions

NOTE: THIS SOFTWARE IS MADE AVAILABLE ON AN AS IS BASIS.
      NO WARRANTY OF ANY KIND IS IMPLIED. SOFTWARE HAS BEEN TESTED 
      on WINDOWS NT. 
\*---------------------------------------------------------*/


//====== TREEVIEW CONTROL =====================================================
typedef TV_ITEM
begin
  INT     mask;             // UINT
  HWND    hItem;            // HTREEITEM 
  INT     state;            // UINT
  INT     stateMask;        // UINT
  POINTER pszText;          // LPTSTR
  int     cchTextMax;       // int
  int     iImage;           // int
  int     iSelectedImage;   // int
  int     cChildren;        // int
  POINTER lParam;           // LPARAM    
end;


typedef TV_INSERTSTRUCT 
begin
  HWND hParent;         // HTREEITEM
  HWND hInsertAfter;    // HTREEITEM
  TV_ITEM item;
end;

/*Image list*/
#define TVSIL_NORMAL            0
#define TVSIL_STATE             2


#define TVI_ROOT                (0xFFFF0000)
#define TVI_FIRST               (0xFFFF0001)
#define TVI_LAST                (0xFFFF0002)
#define TVI_SORT                (0xFFFF0003)

#define TVIF_TEXT               0x0001
#define TVIF_IMAGE              0x0002
#define TVIF_PARAM              0x0004
#define TVIF_STATE              0x0008
#define TVIF_HANDLE             0x0010
#define TVIF_SELECTEDIMAGE      0x0020
#define TVIF_CHILDREN           0x0040
#define TVIF_INTEGRAL           0x0080
#define TVIS_SELECTED           0x0002
#define TVIS_CUT                0x0004
#define TVIS_DROPHILITED        0x0008
#define TVIS_BOLD               0x0010
#define TVIS_EXPANDED           0x0020
#define TVIS_EXPANDEDONCE       0x0040
#define TVIS_EXPANDPARTIAL      0x0080

#define TV_FIRST                0x1100      // TreeView messages

#define TVM_INSERTITEMA         (TV_FIRST + 0)
#define TVM_INSERTITEMW         (TV_FIRST + 50)
#define TVM_INSERTITEM          TVM_INSERTITEMA

#define TVM_SETIMAGELIST        (TV_FIRST + 9)
#define TVM_DELETEITEM          (TV_FIRST + 1)
#define TVM_SETTOOLTIPS         (TV_FIRST + 24)
#define TVM_GETNEXTITEM         (TV_FIRST + 10)

#define TVM_GETITEMA            (TV_FIRST + 12)
#define TVM_GETITEMW            (TV_FIRST + 62)
#define TVM_GETITEM             TVM_GETITEMA

#define TVM_SETITEMA            (TV_FIRST + 13)
#define TVM_SETITEMW            (TV_FIRST + 63)
#define TVM_SETITEM             TVM_SETITEMA

#define TVM_SETTEXTCOLOR        (TV_FIRST + 30)
#define LIGHT_GREY              0x00C0C0C0
#define DARK_GREY               0x00808080
//#define BLACK                   0x00808080

#define TVM_EXPAND              (TV_FIRST + 2)
#define TVE_COLLAPSE            0x0001
#define TVE_EXPAND              0x0002
#define TVE_TOGGLE              0x0003
#define TVE_EXPANDPARTIAL       0x4000
#define TVE_COLLAPSERESET       0x8000


/* 
 * Tree View Styles 
 */
#define TVS_HASBUTTONS          0x0001
#define TVS_HASLINES            0x0002
#define TVS_LINESATROOT         0x0004
#define TVS_EDITLABELS          0x0008
#define TVS_DISABLEDRAGDROP     0x0010
#define TVS_SHOWSELALWAYS       0x0020
#define TVS_RTLREADING          0x0040

#define TVS_NOTOOLTIPS          0x0080
#define TVS_CHECKBOXES          0x0100
#define TVS_TRACKSELECT         0x0200
#define TVS_SINGLEEXPAND        0x0400
#define TVS_INFOTIP             0x0800
#define TVS_FULLROWSELECT       0x1000
#define TVS_NOSCROLL            0x2000
#define TVS_NONEVENHEIGHT       0x4000

#define WC_TREEVIEW "SysTreeView32"

/* 
 * Defines for getting of items 
 */
#define TVGN_ROOT               0x0000
#define TVGN_NEXT               0x0001
#define TVGN_PREVIOUS           0x0002
#define TVGN_PARENT             0x0003
#define TVGN_CHILD              0x0004
#define TVGN_FIRSTVISIBLE       0x0005
#define TVGN_NEXTVISIBLE        0x0006
#define TVGN_PREVIOUSVISIBLE    0x0007
#define TVGN_DROPHILITE         0x0008
#define TVGN_CARET              0x0009


/*
 * OEM Resource Ordinal Numbers
 */
#define OBM_CLOSE           32754
#define OBM_UPARROW         32753
#define OBM_DNARROW         32752
#define OBM_RGARROW         32751
#define OBM_LFARROW         32750
#define OBM_REDUCE          32749
#define OBM_ZOOM            32748
#define OBM_RESTORE         32747
#define OBM_REDUCED         32746
#define OBM_ZOOMD           32745
#define OBM_RESTORED        32744
#define OBM_UPARROWD        32743
#define OBM_DNARROWD        32742
#define OBM_RGARROWD        32741
#define OBM_LFARROWD        32740
#define OBM_MNARROW         32739
#define OBM_COMBO           32738
#define OBM_UPARROWI        32737
#define OBM_DNARROWI        32736
#define OBM_RGARROWI        32735
#define OBM_LFARROWI        32734

#define OBM_OLD_CLOSE       32767
#define OBM_SIZE            32766
#define OBM_OLD_UPARROW     32765
#define OBM_OLD_DNARROW     32764
#define OBM_OLD_RGARROW     32763
#define OBM_OLD_LFARROW     32762
#define OBM_BTSIZE          32761
#define OBM_CHECK           32760
#define OBM_CHECKBOXES      32759
#define OBM_BTNCORNERS      32758
#define OBM_OLD_REDUCE      32757
#define OBM_OLD_ZOOM        32756
#define OBM_OLD_RESTORE     32755


/* Icon/Cursor header */
#define IMAGE_BITMAP        0
#define IMAGE_ICON          1
#define IMAGE_CURSOR        2

#define LR_DEFAULTCOLOR     0x0000
#define LR_MONOCHROME       0x0001
#define LR_COLOR            0x0002
#define LR_COPYRETURNORG    0x0004
#define LR_COPYDELETEORG    0x0008
#define LR_LOADFROMFILE     0x0010
#define LR_LOADTRANSPARENT  0x0020
#define LR_DEFAULTSIZE      0x0040
#define LR_VGACOLOR         0x0080
#define LR_LOADMAP3DCOLORS  0x1000
#define LR_CREATEDIBSECTION 0x2000
#define LR_COPYFROMRESOURCE 0x4000
#define LR_SHARED           0x8000

#define CW_USEDEFAULT       0x80000000


/* Messages*/
#define WM_MOUSEFIRST                   0x0200
#define WM_MOUSEWHEEL                   0x020A
#define WM_MOUSELAST                    0x020A
//#define WM_MOUSELAST                    0x0209

#define WM_KEYFIRST                     0x0100
#define WM_CHAR                         0x0102
#define WM_KEYLAST                      0x0108

/* TREEVIEW IS DEFINES*/
#define BMP_NONE       -1000
#define BMP_KEEPIMAGE  -1001


//====== TREEVIEW FUNCTIONS =====================================================
prototype LIST CtrlTreeIniBitmapFromFile( STRING, INT, BYREF LIST, INT, INT);

prototype INT  CtrlTreeFillFromList(STRING, INT, LIST, INT, INT);
prototype INT  CtrlTreeFillFromMedia(STRING, INT, STRING, INT, INT);

prototype BOOL CtrlTreeDelete( STRING, INT, HWND);
prototype INT  CtrlTreeExpand(STRING, INT, HWND, INT);

prototype VOID CtrlTreeSetItemImage(STRING, INT, HWND, INT, INT);
prototype VOID CtrlTreeSetItemText(STRING, INT, HWND, STRING);

prototype HWND CtrlTreeGetCurSel(STRING, INT);
prototype HWND CtrlTreeGetChild(STRING, INT, HWND);
prototype HWND CtrlTreeGetParent(STRING, INT, HWND);
prototype HWND CtrlTreeGetNextSibling(STRING, INT, HWND);
prototype HWND CtrlTreeGetPrevSibling(STRING, INT, HWND);
prototype HWND CtrlTreeGetRoot(STRING, INT);


//====== TREEVIEW LOCAL FUNCTIONS =====================================================
prototype HWND _TreeGetNextItem(STRING, INT, INT, HWND);
prototype VOID _TreeAddItemAndChildren( HWND, STRING, HWND, HWND, BYREF LIST, INT, INT);
prototype VOID _TreeAddItemAndChildrenMedia( HWND, STRING, HWND, HWND, BYREF LIST, INT, INT);
prototype HWND _TreeAddSingleItem( HWND, BYVAL STRING, HWND, int, int, HWND, int);
prototype BOOL _TreeGetItem(HWND, TV_ITEM POINTER );
prototype BOOL _TreeNextItemIsChild(BYVAL LIST, STRING);


//====== TREEVIEW DLL FUNCTIONS =====================================================
prototype BOOL User32.DestroyWindow(HWND);

prototype HWND User32.LoadBitmap(HWND, POINTER);
prototype HWND user32.LoadImage( HWND, BYREF STRING, INT, int, int, INT);

prototype BOOL gdi32.DeleteObject( HWND );

prototype HWND comctl32.ImageList_Create( int, int, INT, int, int);
prototype INT  comctl32.ImageList_Add( HWND, HWND, HWND, );
prototype INT  comctl32.ImageList_GetImageCount(HWND);

prototype HWND kernel32.LoadLibrary( BYREF STRING );